@@ -1,6 +1,6 @@
 /*
     MIDI Virtual Piano Keyboard
-    Copyright (C) 2008-2010, Pedro Lopez-Cabanillas <plcl@users.sf.net>
+    Copyright (C) 2008-2011, Pedro Lopez-Cabanillas <plcl@users.sf.net>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -30,10 +30,12 @@
 
 const QString BLD_DATE(__DATE__);
 const QString BLD_TIME(__TIME__);
-#if defined(Q_CC_GNU)
+#if defined(Q_CC_GNU) || defined(Q_CC_GCCE)
 const QString CMP_VERSION(__VERSION__);
 #elif defined(Q_CC_MSVC)
 const QString CMP_VERSION(QString::number(_MSC_VER/100.0,'g',2));
+#else
+const QString CMP_VERSION(QString());
 #endif
 
 const QString QSTR_APPNAME("Virtual MIDI Piano Keyboard");
@@ -83,8 +85,17 @@ const QString QSTR_DRUMSCHANNEL("DrumsChannel");
 const QString QSTR_SHORTCUTS("Shortcuts");
 const QString QSTR_LANGUAGE("Language");
 const QString QSTR_VELOCITYCOLOR("VelocityColor");
+const QString QSTR_NETWORKPORT("NetworkPort");
 
+#if defined(SMALL_SCREEN)
+const QString QSTR_VMPKURL("http://vmpk.sourceforge.net/m/");
+const QString QSTR_HELP("hm.html");
+const QString QSTR_HELPL("hm_%1.html");
+#else
 const QString QSTR_VMPKURL("http://vmpk.sourceforge.net");
+const QString QSTR_HELP("help.html");
+const QString QSTR_HELPL("help_%1.html");
+#endif
 
 const char MIDICTLNUMBER[] = "MIDICTLNUMBER\0";
 const char MIDICTLONVALUE[] = "MIDICTLONVAL\0";
@@ -92,7 +103,6 @@ const char MIDICTLOFFVALUE[] = "MIDICTLOFFVAL\0";
 const char SYSEXFILENAME[] = "SYSEXFILENAME\0";
 const char SYSEXFILEDATA[] = "SYSEXFILEDATA\0";
 
-const int TOOLBARLABELMARGIN = 5;
 const int MIDIGMDRUMSCHANNEL = 9;
 const int MIDICHANNELS = 16;
 const int MIDIVELOCITY = 100;
@@ -100,6 +110,21 @@ const int MIDIPAN = 64;
 const int MIDIVOLUME = 100;
 const int MIDIEXPRESSION = 127;
 
+#if defined(SMALL_SCREEN)
+const int DEFAULTNUMBEROFOCTAVES = 2;
+const int TOOLBARLABELMARGIN = 2;
+const int KEYLABELFONTSIZE = 3;
+#else
+const int DEFAULTNUMBEROFOCTAVES = 5;
+const int TOOLBARLABELMARGIN = 5;
+const int KEYLABELFONTSIZE = 7;
+#endif
+const int NETWORKPORTNUMBER = 21928;
+
+#if defined(NETWORK_MIDI)
+extern int g_iUdpPort;
+#endif
+
 #if defined(Q_WS_MAC) && defined(QT_MAC_USE_COCOA)
 #error "This program requires Qt built with the Carbon framework. Sorry."
 #endif
