@@ -1,6 +1,6 @@
 /*
     MIDI Virtual Piano Keyboard
-    Copyright (C) 2008-2010, Pedro Lopez-Cabanillas <plcl@users.sf.net>
+    Copyright (C) 2008-2011, Pedro Lopez-Cabanillas <plcl@users.sf.net>
 
     This library is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,11 +24,13 @@
 #include <QtGui/QShowEvent>
 #include <QtGui/QFileDialog>
 #include <QtGui/QColorDialog>
+#include <QtCore/QDebug>
 
 Preferences::Preferences(QWidget *parent)
     : QDialog(parent),
-    m_numOctaves(5),
+    m_numOctaves(DEFAULTNUMBEROFOCTAVES),
     m_drumsChannel(MIDIGMDRUMSCHANNEL),
+    m_networkPort(NETWORKPORTNUMBER),
     m_grabKb(false),
     m_styledKnobs(true),
     m_alwaysOnTop(false),
@@ -48,6 +50,31 @@ Preferences::Preferences(QWidget *parent)
     connect(ui.btnRawKmap, SIGNAL(clicked()), SLOT(slotOpenRawKeymapFile()));
     QPushButton *btnDefaults = ui.buttonBox->button(QDialogButtonBox::RestoreDefaults);
     connect(btnDefaults, SIGNAL(clicked()), SLOT(slotRestoreDefaults()));
+#if !defined(RAWKBD_SUPPORT)
+    ui.chkRawKeyboard->setVisible(false);
+    ui.lblRawKmap->setVisible(false);
+    ui.txtFileRawKmap->setVisible(false);
+    ui.btnRawKmap->setVisible(false);
+#endif
+#if !defined(NETWORK_MIDI)
+    ui.lblNetworkPort->setVisible(false);
+    ui.txtNetworkPort->setVisible(false);
+#endif
+#if defined(SMALL_SCREEN)
+    ui.chkRawKeyboard->setVisible(false);
+    ui.lblRawKmap->setVisible(false);
+    ui.txtFileRawKmap->setVisible(false);
+    ui.btnRawKmap->setVisible(false);
+    ui.lblKmap->setVisible(false);
+    ui.txtFileKmap->setVisible(false);
+    ui.btnKmap->setVisible(false);
+    ui.chkAlwaysOnTop->setVisible(false);
+    ui.chkGrabKb->setVisible(false);
+    setWindowState(Qt::WindowActive | Qt::WindowMaximized);
+#else
+    setMinimumSize(480,320);
+    adjustSize();
+#endif
 }
 
 void Preferences::showEvent ( QShowEvent *event )
@@ -60,6 +87,7 @@ void Preferences::showEvent ( QShowEvent *event )
         ui.chkAlwaysOnTop->setChecked( m_alwaysOnTop );
         ui.chkRawKeyboard->setChecked( m_rawKeyboard );
         ui.chkVelocityColor->setChecked( m_velocityColor );
+        ui.txtNetworkPort->setText( QString::number( m_networkPort ));
         if (!m_keyPressedColor.isValid()) {
             setKeyPressedColor(QApplication::palette().highlight().color());
         }
@@ -85,6 +113,7 @@ void Preferences::apply()
          ui.txtFileInstrument->text() == QSTR_DEFAULT )
         m_insFileName = QSTR_DEFAULT;
     m_drumsChannel = ui.cboDrumsChannel->currentIndex() - 1;
+    m_networkPort = ui.txtNetworkPort->text().toInt();
 }
 
 void Preferences::accept()
@@ -150,6 +179,8 @@ void Preferences::setInstrumentsFileName( const QString fileName )
             m_insFileName.clear();
             ui.txtFileInstrument->setText(m_insFileName);
         }
+    } else {
+        qDebug() << "file" << fileName << "not readable.";
     }
 }
 
@@ -231,12 +262,13 @@ void Preferences::restoreDefaults()
     ui.chkGrabKb->setChecked(false);
     ui.chkRawKeyboard->setChecked(false);
     ui.chkStyledKnobs->setChecked(true);
-    ui.spinNumOctaves->setValue(5);
+    ui.spinNumOctaves->setValue(DEFAULTNUMBEROFOCTAVES);
     ui.txtFileKmap->setText(QSTR_DEFAULT);
     ui.txtFileRawKmap->setText(QSTR_DEFAULT);
     ui.chkVelocityColor->setChecked(true);
     setInstrumentsFileName(VPiano::dataDirectory() + QSTR_DEFAULTINS);
     ui.cboInstrument->setCurrentIndex(0);
+    ui.txtNetworkPort->setText(QString::number(NETWORKPORTNUMBER));
 }
 
 void Preferences::slotRestoreDefaults()
