@@ -1,6 +1,6 @@
 /*
     Virtual Piano Widget for Qt4
-    Copyright (C) 2008-2010, Pedro Lopez-Cabanillas <plcl@users.sf.net>
+    Copyright (C) 2008-2011, Pedro Lopez-Cabanillas <plcl@users.sf.net>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,6 +16,8 @@
     with this program; If not, see <http://www.gnu.org/licenses/>.
 */
 
+#if defined(RAWKBD_SUPPORT)
+
 #include "rawkeybdapp.h"
 
 #if defined(Q_WS_X11)
@@ -147,3 +149,5 @@ bool RawKeybdApp::macEventFilter( EventHandlerCallRef /*caller*/, EventRef event
     return false;
 }
 #endif  // Q_WS_MAC
+
+#endif // RAWKBD_SUPPORT
