@@ -1,6 +1,6 @@
 /*
     MIDI Virtual Piano Keyboard
-    Copyright (C) 2008-2010, Pedro Lopez-Cabanillas <plcl@users.sf.net>
+    Copyright (C) 2008-2011, Pedro Lopez-Cabanillas <plcl@users.sf.net>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -28,9 +28,12 @@
 #include "about.h"
 #include "preferences.h"
 #include "midisetup.h"
-#include "kmapdialog.h"
 #include "events.h"
+
+#if !defined(SMALL_SCREEN)
+#include "kmapdialog.h"
 #include "shortcutdialog.h"
+#endif
 
 #if ENABLE_DBUS
 #include "vmpk_adaptor.h"
@@ -49,6 +52,10 @@
 #include <QtGui/QDial>
 #include <QtGui/QToolButton>
 #include <QtGui/QToolTip>
+#include <QtGui/QVBoxLayout>
+#include <QtGui/QTextBrowser>
+#include <QtGui/QDialogButtonBox>
+#include <QtGui/QDialog>
 #include <QtCore/QUrl>
 #include <QtCore/QString>
 #include <QtCore/QSettings>
@@ -89,7 +96,6 @@ VPiano::VPiano( QWidget * parent, Qt::WindowFlags flags )
     QCoreApplication::installTranslator(m_trp);
     ui.setupUi(this);
     initLanguages();
-
     connect(ui.actionAbout, SIGNAL(triggered()), SLOT(slotAbout()));
     connect(ui.actionAboutQt, SIGNAL(triggered()), SLOT(slotAboutQt()));
     connect(ui.actionAboutTranslation, SIGNAL(triggered()), SLOT(slotAboutTranslation()));
@@ -102,6 +108,28 @@ VPiano::VPiano( QWidget * parent, Qt::WindowFlags flags )
     connect(ui.actionEditExtraControls, SIGNAL(triggered()), SLOT(slotEditExtraControls()));
     connect(ui.actionNoteNames, SIGNAL(triggered()), SLOT(slotShowNoteNames()));
     connect(ui.actionShortcuts, SIGNAL(triggered()), SLOT(slotShortcuts()));
+    // Toolbars actions: toggle view
+    connect(ui.toolBarNotes->toggleViewAction(), SIGNAL(toggled(bool)),
+            ui.actionNotes, SLOT(setChecked(bool)));
+    connect(ui.toolBarControllers->toggleViewAction(), SIGNAL(toggled(bool)),
+            ui.actionControllers, SLOT(setChecked(bool)));
+    connect(ui.toolBarBender->toggleViewAction(), SIGNAL(toggled(bool)),
+            ui.actionBender, SLOT(setChecked(bool)));
+    connect(ui.toolBarPrograms->toggleViewAction(), SIGNAL(toggled(bool)),
+            ui.actionPrograms, SLOT(setChecked(bool)));
+    connect(ui.toolBarExtra->toggleViewAction(), SIGNAL(toggled(bool)),
+            ui.actionExtraControls, SLOT(setChecked(bool)));
+#if defined(SMALL_SCREEN)
+    ui.toolBarControllers->hide();
+    ui.toolBarBender->hide();
+    ui.toolBarExtra->hide();
+    //ui.toolBarNotes->hide();
+    //ui.toolBarPrograms->hide();
+    ui.actionEditKM->setVisible(false);
+    ui.actionShortcuts->setVisible(false);
+    ui.actionStatusBar->setVisible(false);
+    setWindowTitle("VMPK " + PGM_VERSION);
+#endif
     ui.pianokeybd->setPianoHandler(this);
     initialization();
 }
@@ -138,6 +166,7 @@ void VPiano::initialization()
         applyConnections();
         applyInitialSettings();
         initExtraControllers();
+        activateWindow();
     }
 }
 
@@ -206,7 +235,7 @@ bool VPiano::initMidi()
     try {
         m_midiout = new RtMidiOut(QSTR_VMPKOUTPUT.toStdString());
         m_midiin = new RtMidiIn(QSTR_VMPKINPUT.toStdString());
-#if !defined(__LINUX_ALSASEQ__) && !defined(__MACOSX_CORE__)
+#if !defined(__LINUX_ALSASEQ__) && !defined(__MACOSX_CORE__) && !defined(__LINUX_JACK__)
         int nOutPorts = m_midiout->getPortCount();
         if (nOutPorts == 0) {
             delete m_midiout;
@@ -221,7 +250,7 @@ bool VPiano::initMidi()
             m_midiin = 0;
         }
 #endif
-#if defined(__LINUX_ALSASEQ__) || defined(__MACOSX_CORE__)
+#if defined(__LINUX_ALSASEQ__) || defined(__MACOSX_CORE__) || defined(__LINUX_JACK__)
         if (m_midiout != 0)
             m_midiout->openVirtualPort(QSTR_VMPKOUTPUT.toStdString());
         if (m_midiin != 0)
@@ -249,7 +278,15 @@ void VPiano::initToolBars()
     m_dialStyle = new ClassicStyle();
     m_dialStyle->setParent(this);
     // Notes tool bar
-    m_lblChannel = new QLabel(tr("Channel:"), this);
+    QWidget *w = ui.toolBarNotes->widgetForAction(ui.actionPanic);
+    w->setMaximumWidth(120);
+    m_lblChannel = new QLabel(
+#if defined(SMALL_SCREEN)
+        tr("Chan:")
+#else
+        tr("Channel:")
+#endif
+    , this);
     ui.toolBarNotes->addWidget(m_lblChannel);
     m_lblChannel->setMargin(TOOLBARLABELMARGIN);
     m_sboxChannel = new QSpinBox(this);
@@ -258,7 +295,13 @@ void VPiano::initToolBars()
     m_sboxChannel->setValue(m_channel + 1);
     m_sboxChannel->setFocusPolicy(Qt::NoFocus);
     ui.toolBarNotes->addWidget(m_sboxChannel);
-    m_lblBaseOctave = new QLabel(tr("Base Octave:"), this);
+    m_lblBaseOctave = new QLabel(
+#if defined(SMALL_SCREEN)
+        tr("Oct:")
+#else
+        tr("Base Octave:")
+#endif
+    , this);
     ui.toolBarNotes->addWidget(m_lblBaseOctave);
     m_lblBaseOctave->setMargin(TOOLBARLABELMARGIN);
     m_sboxOctave = new QSpinBox(this);
@@ -267,7 +310,13 @@ void VPiano::initToolBars()
     m_sboxOctave->setValue(m_baseOctave);
     m_sboxOctave->setFocusPolicy(Qt::NoFocus);
     ui.toolBarNotes->addWidget(m_sboxOctave);
-    m_lblTranspose = new QLabel(tr("Transpose:"), this);
+    m_lblTranspose = new QLabel(
+#if defined(SMALL_SCREEN)
+        tr("Trans:")
+#else
+        tr("Transpose:")
+#endif
+    , this);
     ui.toolBarNotes->addWidget(m_lblTranspose);
     m_lblTranspose->setMargin(TOOLBARLABELMARGIN);
     m_sboxTranspose = new QSpinBox(this);
@@ -276,7 +325,13 @@ void VPiano::initToolBars()
     m_sboxTranspose->setValue(m_transpose);
     m_sboxTranspose->setFocusPolicy(Qt::NoFocus);
     ui.toolBarNotes->addWidget(m_sboxTranspose);
-    m_lblVelocity = new QLabel(tr("Velocity:"), this);
+    m_lblVelocity = new QLabel(
+#if defined(SMALL_SCREEN)
+        tr("Vel:")
+#else
+        tr("Velocity:")
+#endif
+    , this);
     ui.toolBarNotes->addWidget(m_lblVelocity);
     m_lblVelocity->setMargin(TOOLBARLABELMARGIN);
     m_Velocity = new Knob(this);
@@ -376,17 +431,6 @@ void VPiano::initToolBars()
              SLOT(slotComboBankActivated(int)) );
     connect( m_comboProg, SIGNAL(activated(int)),
              SLOT(slotComboProgActivated(int)) );
-    // Toolbars actions: toggle view
-    connect(ui.toolBarNotes->toggleViewAction(), SIGNAL(toggled(bool)),
-            ui.actionNotes, SLOT(setChecked(bool)));
-    connect(ui.toolBarControllers->toggleViewAction(), SIGNAL(toggled(bool)),
-            ui.actionControllers, SLOT(setChecked(bool)));
-    connect(ui.toolBarBender->toggleViewAction(), SIGNAL(toggled(bool)),
-            ui.actionBender, SLOT(setChecked(bool)));
-    connect(ui.toolBarPrograms->toggleViewAction(), SIGNAL(toggled(bool)),
-            ui.actionPrograms, SLOT(setChecked(bool)));
-    connect(ui.toolBarExtra->toggleViewAction(), SIGNAL(toggled(bool)),
-            ui.actionExtraControls, SLOT(setChecked(bool)));
     // Toolbars actions: buttons
     connect( ui.actionPanic, SIGNAL(triggered()),
              SLOT(slotPanic()));
@@ -560,7 +604,7 @@ void VPiano::readSettings()
     m_velocity = settings.value(QSTR_VELOCITY, MIDIVELOCITY).toInt();
     m_baseOctave = settings.value(QSTR_BASEOCTAVE, 3).toInt();
     m_transpose = settings.value(QSTR_TRANSPOSE, 0).toInt();
-    int num_octaves = settings.value(QSTR_NUMOCTAVES, 5).toInt();
+    int num_octaves = settings.value(QSTR_NUMOCTAVES, DEFAULTNUMBEROFOCTAVES).toInt();
     QString insFileName = settings.value(QSTR_INSTRUMENTSDEFINITION).toString();
     QString insName = settings.value(QSTR_INSTRUMENTNAME).toString();
     QColor keyColor = settings.value(QSTR_KEYPRESSEDCOLOR, QColor()).value<QColor>();
@@ -571,8 +615,14 @@ void VPiano::readSettings()
     bool showStatusBar = settings.value(QSTR_SHOWSTATUSBAR, false).toBool();
     bool velocityColor = settings.value(QSTR_VELOCITYCOLOR, true).toBool();
     int drumsChannel = settings.value(QSTR_DRUMSCHANNEL, MIDIGMDRUMSCHANNEL).toInt();
+#if defined(NETWORK_MIDI)
+    g_iUdpPort = settings.value(QSTR_NETWORKPORT, NETWORKPORTNUMBER).toInt();
+#endif
     settings.endGroup();
 
+#if defined(NETWORK_MIDI)
+    dlgPreferences()->setNetworkPort(g_iUdpPort);
+#endif
     dlgPreferences()->setNumOctaves(num_octaves);
     dlgPreferences()->setDrumsChannel(drumsChannel);
     dlgPreferences()->setKeyPressedColor(keyColor);
@@ -594,8 +644,10 @@ void VPiano::readSettings()
     settings.beginGroup(QSTR_CONNECTIONS);
     bool inEnabled = settings.value(QSTR_INENABLED, true).toBool();
     bool thruEnabled = settings.value(QSTR_THRUENABLED, false).toBool();
+#if !defined(NETWORK_MIDI)
     QString in_port = settings.value(QSTR_INPORT).toString();
     QString out_port = settings.value(QSTR_OUTPORT).toString();
+#endif
     settings.endGroup();
 #if defined(__LINUX_ALSASEQ__) || defined(__MACOSX_CORE__)
     inEnabled = true;
@@ -606,10 +658,13 @@ void VPiano::readSettings()
     } else {
         dlgMidiSetup()->setInputEnabled(inEnabled);
         dlgMidiSetup()->setThruEnabled(thruEnabled);
+#if !defined(NETWORK_MIDI)
         dlgMidiSetup()->setCurrentInput(in_port);
+#endif
     }
+#if !defined(NETWORK_MIDI)
     dlgMidiSetup()->setCurrentOutput(out_port);
-
+#endif
     settings.beginGroup(QSTR_KEYBOARD);
     bool rawKeyboard = settings.value(QSTR_RAWKEYBOARDMODE, false).toBool();
     QString mapFile = settings.value(QSTR_MAPFILE, QSTR_DEFAULT).toString();
@@ -698,13 +753,18 @@ void VPiano::writeSettings()
     settings.setValue(QSTR_SHOWSTATUSBAR, ui.actionStatusBar->isChecked());
     settings.setValue(QSTR_DRUMSCHANNEL, dlgPreferences()->getDrumsChannel());
     settings.setValue(QSTR_VELOCITYCOLOR, dlgPreferences()->getVelocityColor());
+#if defined(NETWORK_MIDI)
+    settings.setValue(QSTR_NETWORKPORT, dlgPreferences()->getNetworkPort());
+#endif
     settings.endGroup();
 
     settings.beginGroup(QSTR_CONNECTIONS);
     settings.setValue(QSTR_INENABLED, dlgMidiSetup()->inputIsEnabled());
     settings.setValue(QSTR_THRUENABLED, dlgMidiSetup()->thruIsEnabled());
+#if !defined(NETWORK_MIDI)
     settings.setValue(QSTR_INPORT,  dlgMidiSetup()->selectedInputName());
     settings.setValue(QSTR_OUTPORT, dlgMidiSetup()->selectedOutputName());
+#endif
     settings.endGroup();
 
     settings.beginGroup(QSTR_KEYBOARD);
@@ -761,6 +821,7 @@ void VPiano::writeSettings()
 
 void VPiano::closeEvent( QCloseEvent *event )
 {
+    //qDebug() << "closeEvent:" << event->type();
     if (m_initialized)
         writeSettings();
     event->accept();
@@ -768,6 +829,7 @@ void VPiano::closeEvent( QCloseEvent *event )
 
 void VPiano::customEvent ( QEvent *event )
 {
+    //qDebug() << "customEvent:" << event->type();
     if ( event->type() == NoteOnEventType ) {
         NoteOnEvent *ev = static_cast<NoteOnEvent*>(event);
         int n = ev->getNote();
@@ -842,16 +904,22 @@ void VPiano::customEvent ( QEvent *event )
 
 void VPiano::showEvent ( QShowEvent *event )
 {
+    //qDebug() << "showEvent:" << event->type();
+    QMainWindow::showEvent(event);
+#if !defined(SMALL_SCREEN)
     if (m_initialized) {
-        QMainWindow::showEvent(event);
         ui.pianokeybd->setFocus();
         grabKb();
     }
+#endif
 }
 
 void VPiano::hideEvent( QHideEvent *event )
 {
+    //qDebug() << "hideEvent:" << event->type();
+#if !defined(SMALL_SCREEN)
     releaseKb();
+#endif
     QMainWindow::hideEvent(event);
 }
 
@@ -875,12 +943,12 @@ void VPiano::sendMessageWrapper(std::vector<unsigned char> *message) const
     }
 }
 
-void VPiano::sendNoteOn(const int midiNote)
+void VPiano::sendNoteOn(const int midiNote, const int vel)
 {
     std::vector<unsigned char> message;
     if ((midiNote & MASK_SAFETY) == midiNote) {
         unsigned char chan = static_cast<unsigned char>(m_channel);
-        unsigned char vel = static_cast<unsigned char>(m_velocity);
+        //unsigned char vel = static_cast<unsigned char>(m_velocity);
         // Note On: 0x90 + channel, note, vel
         message.push_back(STATUS_NOTEON + (chan & MASK_CHANNEL));
         message.push_back(midiNote & MASK_SAFETY);
@@ -889,20 +957,20 @@ void VPiano::sendNoteOn(const int midiNote)
     }
 }
 
-void VPiano::noteOn(const int midiNote)
+void VPiano::noteOn(const int midiNote, const int vel)
 {
-    sendNoteOn(midiNote);
+    sendNoteOn(midiNote, vel);
 #ifdef ENABLE_DBUS
     emit event_noteon(midiNote);
 #endif
 }
 
-void VPiano::sendNoteOff(const int midiNote)
+void VPiano::sendNoteOff(const int midiNote, const int vel)
 {
     std::vector<unsigned char> message;
     if ((midiNote & MASK_SAFETY) == midiNote) {
         unsigned char chan = static_cast<unsigned char>(m_channel);
-        unsigned char vel = static_cast<unsigned char>(m_velocity);
+        //unsigned char vel = static_cast<unsigned char>(m_velocity);
         // Note Off: 0x80 + channel, note, vel
         message.push_back(STATUS_NOTEOFF + (chan & MASK_CHANNEL));
         message.push_back(midiNote & MASK_SAFETY);
@@ -911,9 +979,9 @@ void VPiano::sendNoteOff(const int midiNote)
     }
 }
 
-void VPiano::noteOff(const int midiNote)
+void VPiano::noteOff(const int midiNote, const int vel)
 {
-    sendNoteOff(midiNote);
+    sendNoteOff(midiNote, vel);
 #ifdef ENABLE_DBUS
     emit event_noteoff(midiNote);
 #endif
@@ -1281,6 +1349,10 @@ void VPiano::applyPreferences()
     ui.pianokeybd->setRawKeyboardMode(dlgPreferences()->getRawKeyboard());
     ui.pianokeybd->setVelocity(dlgPreferences()->getVelocityColor() ? m_velocity : MIDIVELOCITY );
 
+#if defined(NETWORK_MIDI)
+    g_iUdpPort = dlgPreferences()->getNetworkPort();
+#endif
+
     KeyboardMap* map = dlgPreferences()->getKeyboardMap();
     if (!map->getFileName().isEmpty() && map->getFileName() != QSTR_DEFAULT )
         ui.pianokeybd->setKeyboardMap(map);
@@ -1364,30 +1436,38 @@ void VPiano::applyInitialSettings()
 
 void VPiano::slotPreferences()
 {
+#if defined(NETWORK_MIDI)
+    int old_udpPort = g_iUdpPort;
+#endif
     releaseKb();
     if (dlgPreferences()->exec() == QDialog::Accepted) {
         applyPreferences();
+#if defined(NETWORK_MIDI)
+        if (old_udpPort != g_iUdpPort) {
+            applyConnections();
+        }
+#endif
     }
     grabKb();
 }
 
 QString VPiano::dataDirectory()
 {
-#ifdef Q_OS_WIN32
+#if defined(Q_OS_WIN32)
     return QApplication::applicationDirPath() + "/";
-#endif
-#ifdef Q_OS_LINUX
+#elif defined(Q_OS_LINUX)
     return QApplication::applicationDirPath() + "/../share/vmpk/";
-#endif
-#ifdef Q_OS_DARWIN
+#elif defined(Q_OS_DARWIN)
     return QApplication::applicationDirPath() + "/../Resources/";
+#elif defined(Q_OS_SYMBIAN)
+    return QApplication::applicationDirPath() + "/";
 #endif
     return QString();
 }
 
 QString VPiano::localeDirectory()
 {
-#ifdef Q_OS_LINUX
+#if defined(Q_OS_LINUX)
     return VPiano::dataDirectory() + "locale/";
 #else
     return VPiano::dataDirectory();
@@ -1396,6 +1476,7 @@ QString VPiano::localeDirectory()
 
 void VPiano::slotEditKeyboardMap()
 {
+#if !defined(SMALL_SCREEN)
     KeyboardMap* map;
     releaseKb();
     if (dlgPreferences()->getRawKeyboard())
@@ -1411,6 +1492,7 @@ void VPiano::slotEditKeyboardMap()
             ui.pianokeybd->setKeyboardMap(map);
     }
     grabKb();
+#endif
 }
 
 void VPiano::populatePrograms(int bank)
@@ -1604,34 +1686,61 @@ void VPiano::slotComboControlCurrentIndexChanged(const int index)
 
 void VPiano::grabKb()
 {
+#if !defined(SMALL_SCREEN)
     if (dlgPreferences()->getGrabKeyboard()) {
         ui.pianokeybd->grabKeyboard();
     }
     ui.pianokeybd->setRawKeyboardMode(dlgPreferences()->getRawKeyboard());
+#endif
 }
 
 void VPiano::releaseKb()
 {
+#if !defined(SMALL_SCREEN)
     if (dlgPreferences()->getGrabKeyboard()) {
         ui.pianokeybd->releaseKeyboard();
     }
     ui.pianokeybd->setRawKeyboardMode(false);
+#endif
 }
 
+class HelpDialog : public QDialog
+{
+public:
+    HelpDialog(const QUrl &document, QWidget *parent = 0) : QDialog(parent)
+    {
+        setWindowState(Qt::WindowMaximized | Qt::WindowActive);
+        QVBoxLayout *layout = new QVBoxLayout(this);
+        QTextBrowser *browser = new QTextBrowser(this);
+        layout->addWidget(browser);
+        QDialogButtonBox *buttonBox = new QDialogButtonBox(this);
+        buttonBox->setOrientation(Qt::Horizontal);
+        buttonBox->setStandardButtons(QDialogButtonBox::Ok);
+        layout->addWidget(buttonBox);
+        browser->setSource(document);
+        connect(buttonBox, SIGNAL(accepted()), SLOT(close()));
+    }
+};
+
 void VPiano::slotHelpContents()
 {
     QStringList hlps;
     QLocale loc(configuredLanguage());
     QStringList lc = loc.name().split("_");
-    hlps += QString("help_%1.html").arg(loc.name());
+    hlps += QSTR_HELPL.arg(loc.name());
     if (lc.count() > 1)
-        hlps += QString("help_%1.html").arg(lc[0]);
-    hlps += "help.html";
+        hlps += QSTR_HELPL.arg(lc[0]);
+    hlps += QSTR_HELP;
+    QDir hlpDir(VPiano::dataDirectory());
     foreach(const QString& hlp_name, hlps) {
-        QString fullName = VPiano::dataDirectory() + hlp_name;
-        if (QFile::exists(fullName)) {
-            QUrl url = QUrl::fromLocalFile(fullName);
+        if (hlpDir.exists(hlp_name)) {
+            QUrl url = QUrl::fromLocalFile(hlpDir.absoluteFilePath(hlp_name));
+#if defined(SMALL_SCREEN)
+            HelpDialog hlpDlg(url, this);
+            hlpDlg.exec();
+#else
             QDesktopServices::openUrl(url);
+#endif
             return;
         }
     }
@@ -1707,9 +1816,11 @@ MidiSetup* VPiano::dlgMidiSetup()
 
 KMapDialog* VPiano::dlgKeyMap()
 {
+#if !defined(SMALL_SCREEN)
     if (m_dlgKeyMap == 0) {
         m_dlgKeyMap = new KMapDialog(this);
     }
+#endif
     return m_dlgKeyMap;
 }
 
@@ -1809,14 +1920,14 @@ void VPiano::connect_thru(bool value)
 
 void VPiano::noteoff(int note)
 {
-    sendNoteOff(note);
+    sendNoteOff(note, 0);
     NoteOffEvent *ev = new NoteOffEvent(note, 0);
     QApplication::postEvent(this, ev);
 }
 
 void VPiano::noteon(int note)
 {
-    sendNoteOn(note);
+    sendNoteOn(note, m_velocity);
     NoteOnEvent *ev = new NoteOnEvent(note, m_velocity);
     QApplication::postEvent(this, ev);
 }
@@ -1869,10 +1980,12 @@ void VPiano::pitchwheel(int value)
 
 void VPiano::slotShortcuts()
 {
+#if !defined(SMALL_SCREEN)
     ShortcutDialog shcutDlg(findChildren<QAction*>());
     releaseKb();
     shcutDlg.exec();
     grabKb();
+#endif
 }
 
 void VPiano::slotBankNext()
@@ -2049,8 +2162,10 @@ void VPiano::retranslateUi()
         m_dlgPreferences->retranslateUi();
     if (m_dlgMidiSetup != 0)
         m_dlgMidiSetup->retranslateUi();
+#if !defined(SMALL_SCREEN)
     if (m_dlgKeyMap != 0)
         m_dlgKeyMap->retranslateUi();
+#endif
     if (m_dlgExtra != 0)
         m_dlgExtra->retranslateUi();
     if (m_dlgRiffImport != 0)
@@ -2065,20 +2180,54 @@ void VPiano::initLanguages()
     m_supportedLangs.insert(QLatin1String("en"), tr("English"));
     m_supportedLangs.insert(QLatin1String("es"), tr("Spanish"));
     m_supportedLangs.insert(QLatin1String("fr"), tr("French"));
+    m_supportedLangs.insert(QLatin1String("nl"), tr("Dutch"));
     m_supportedLangs.insert(QLatin1String("ru"), tr("Russian"));
-    m_supportedLangs.insert(QLatin1String("tr"), tr("Turkish"));
+    m_supportedLangs.insert(QLatin1String("sv"), tr("Swedish"));
     m_supportedLangs.insert(QLatin1String("zh_CN"), tr("Chinese"));
 }
 
 void VPiano::retranslateToolbars()
 {
+    m_lblChannel->setText(
+#if defined(SMALL_SCREEN)
+        tr("Chan:")
+#else
+        tr("Channel:")
+#endif
+    );
+    m_lblBaseOctave->setText(
+#if defined(SMALL_SCREEN)
+        tr("Oct:")
+#else
+        tr("Base Octave:")
+#endif
+    );
+    m_lblTranspose->setText(
+#if defined(SMALL_SCREEN)
+        tr("Trans:")
+#else
+        tr("Transpose:")
+#endif
+    );
+    m_lblVelocity->setText(
+#if defined(SMALL_SCREEN)
+        tr("Vel:")
+#else
+        tr("Velocity:")
+#endif
+    );
     m_lblBank->setText(tr("Bank:"));
-    m_lblBaseOctave->setText(tr("Base Octave:"));
     m_lblBender->setText(tr("Bender:"));
-    m_lblChannel->setText(tr("Channel:"));
     m_lblControl->setText(tr("Control:"));
     m_lblProgram->setText(tr("Program:"));
-    m_lblTranspose->setText(tr("Transpose:"));
     m_lblValue->setText(tr("Value:"));
-    m_lblVelocity->setText(tr("Velocity:"));
+}
+
+QMenu * VPiano::createPopupMenu ()
+{
+#if defined(SMALL_SCREEN)
+    return 0;
+#else
+    return QMainWindow::createPopupMenu();
+#endif
 }
