@@ -1,6 +1,6 @@
 /*
     MIDI Virtual Piano Keyboard
-    Copyright (C) 2008-2010, Pedro Lopez-Cabanillas <plcl@users.sf.net>
+    Copyright (C) 2008-2011, Pedro Lopez-Cabanillas <plcl@users.sf.net>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -51,10 +51,11 @@ public:
     void midiThru(std::vector<unsigned char> *message) const;
     bool isInitialized() const { return m_initialized; }
     void retranslateUi();
+    QMenu *createPopupMenu ();
 
     // PianoHandler methods
-    void noteOn(const int midiNote);
-    void noteOff(const int midiNote);
+    void noteOn(const int midiNote, const int vel);
+    void noteOff(const int midiNote, const int vel);
 
     // static methods
     static QString dataDirectory();
@@ -163,8 +164,8 @@ private:
     void allNotesOff();
     void sendController(const int controller, const int value);
     void sendBankChange(const int bank);
-    void sendNoteOn(const int midiNote);
-    void sendNoteOff(const int midiNote);
+    void sendNoteOn(const int midiNote, const int vel);
+    void sendNoteOff(const int midiNote, const int vel);
     void sendProgramChange(const int program);
     void sendBender(const int value);
     void sendPolyKeyPress(const int note, const int value);
